(function($) {
  /**
   * OpenEDU Collapsible.
   */
  Drupal.OpenEDUJSCollapsibleLists = {
    // Customize title.
    customTitle: function(title) {
      title.each(function(k, v) {
        var $this = $(v),
            text = $this.text();

        // Create new title/link wrapper.
        $this
          .wrap('<div class="title-wrapper"></div>')
          .before(text + ' ')
          .text(Drupal.t('view course'));
      });
    },
    // Attach behavior.
    attach: function(selector) {
      var that = this,
          selectors = selector.split('|') || null,
          classes = {
            processed: 'processed',
            expanded: 'expanded',
            wrapper: selectors[0] || null,
            item: selectors[1] || null,
            title: selectors[2] || null,
            content: selectors[3] || null
          };

      if (!classes.wrapper || !classes.item || !classes.title ||
        !classes.content || $(classes.wrapper).hasClass(classes.processed)) {
        return;
      }

      // Enforce all content fields are hidden.
      $(classes.wrapper + ' ' + classes.content).hide();

      // Add processed class to wrapper to avoid been unnecessary reloaded.
      $(classes.wrapper).addClass(classes.processed);

      // Manage link to create clicable icon.
      that.customTitle( $(classes.wrapper + ' ' + classes.title).find('a') );

      // Create click behavior.
      $(classes.wrapper + ' ' + classes.title).on('click', function(e) {
        // Set vars.
        var $target = $(e.target),
            $that = $(this),
            $parent = $that.closest(classes.item),
            $content = $parent.find(classes.content);

        // Check if link was clicked.
        if ($target.is('a')) {
          return;
        }

        // Check if parent is available.
        if ($content.length) {
          // Toggle content.
          $content.slideToggle();

          // Append class to title.
          $that.toggleClass(classes.expanded);
        }
      });
    }
  };

  /**
   * OpenEDU Collapsible Cards.
   */
  Drupal.OpenEDUJSCollapsibleCards = function(items, card_class) {
    // Set vars.
    var classes = {
          icon: 'openedu-collapsible-icon',
          processed: 'processed',
          group_header: 'group-header',
          group_collapsible: 'group-collapsible',
          group_row_first: 'group-first-row',
          group_row_second: 'group-second-row',
          group_row_third: 'group-third-row'
        };

    // If available.
    if (items.length) {
      items.each(function() {
        // Set vars.
        var $this = $(this);

        // Hide 3rd row if CSS didn't yet.
          $this.find('.' + classes.group_row_third).hide();

        // Prepend icon on collapsible groups.
        $this.find('.' + classes.group_collapsible)
          .prepend($('<span/>').attr({
              class: classes.icon
            }).text('(X)')
          )
          // When clicking on new placed icon:
          //  #1. Toggle icon class.
          //  #2. Toggle wrapper class.
          //  #3. Toggle group_header
          //  #4. Toggle group_row_second && group_row_third
          .on('click', '.' + classes.icon, function(e) {
            e.preventDefault();

            // Set vars.
            var $this = $(this),
                parents = $this.parents(card_class);

            // #1.
            $this.toggleClass(classes.processed);

            // #2.
            parents.toggleClass(classes.processed);

            // #3.
            parents.find('.' + classes.group_header).slideToggle();

            // #4.
            parents.find('.' + classes.group_row_second).slideToggle();
            parents.find('.' + classes.group_row_third).slideToggle();
          });
      });
    }
  };

  /**
   * OpenEDU Collapsible Cards Behaviors.
   */
  Drupal.behaviors.OpenEDUJSCollapsible = {
    attach: function (context) {
      var collapsibleCards = Drupal.settings.OpenEDUJSCollapsible.cards.selectors,
          collapsibleLists = Drupal.settings.OpenEDUJSCollapsible.lists.selectors;

      // Collapsible Cards.
      if (collapsibleCards.length) {
        $.each(collapsibleCards, function(k, v) {
          // Set vars.
          var items = $(v),
              classes = v.split(/[\s,]+/) || null,
              card_class = classes[classes.length-1] || null;

          // Apply effect.
          Drupal.OpenEDUJSCollapsibleCards(items, card_class);
        });
      }

      // Collapsible Lists.
      if (collapsibleLists.length) {
        $.each(collapsibleLists, function(k, v) {
          Drupal.OpenEDUJSCollapsibleLists.attach(v);
        });
      }

    }
  };

})(jQuery);